@@ -32,7 +32,7 @@ if TYPE_CHECKING:
     from chython import MoleculeContainer
 
 
-class StandardizeMolecule:
+class Standardize:
     __slots__ = ()
 
     def canonicalize(self: 'MoleculeContainer', *, logging=False) -> \
@@ -66,9 +66,7 @@ class StandardizeMolecule:
 
         :param logging: return list of fixed atoms with matched rules.
         """
-        neutralized = self.__fix_resonance(logging=logging)
-        if neutralized:
-            self.flush_cache()
+        neutralized = self.fix_resonance(logging=logging)
         log = self.__standardize(double_rules)
         log.extend(self.__standardize(double_rules))  # double shot rules for overlapped groups
         log.extend(self.__standardize(single_rules))
@@ -288,8 +286,7 @@ class StandardizeMolecule:
             except TypeError:
                 raise ValenceError(f'atom {{{n}}} has valence error')
 
-        if return_maps:
-            log = []
+        log = []
         if to_add:
             bonds = self._bonds
             m = start_map
@@ -297,8 +294,7 @@ class StandardizeMolecule:
                 m = self.add_atom(H(), _map=m)
                 bonds[n][m] = bonds[m][n] = Bond(1)
                 hydrogens[n] = 0
-                if return_maps:
-                    log.append((n, m))
+                log.append((n, m))
                 m += 1
 
             if fix_stereo:
@@ -370,14 +366,10 @@ class StandardizeMolecule:
 
         log = []
         flush = False
-        for r, (pattern, atom_fix, bonds_fix) in enumerate(rules):
+        for r, (pattern, atom_fix, bonds_fix, any_atoms) in enumerate(rules):
             hs = set()
             seen = set()
-            # collect constrain Any-atoms
-            any_atoms = [n for n, a in pattern.atoms() if a.atomic_symbol == 'A' and n not in atom_fix]
-            # AnyMetal can match multiple times
-            any_atoms.extend(n for n, a in pattern.atoms() if a.atomic_symbol == 'M')
-            for mapping in pattern.get_mapping(self, optimize=False, automorphism_filter=False):
+            for mapping in pattern.get_mapping(self, automorphism_filter=False):
                 match = set(mapping.values())
                 if not match.isdisjoint(seen):  # skip intersected groups
                     continue
@@ -411,136 +403,30 @@ class StandardizeMolecule:
                             if b != 8:
                                 flush = True
                     log.append((tuple(match), r, str(pattern)))
+
+            if not hs:  # not matched
+                continue
             # flush cache only for changed atoms.
             if flush:  # neighbors count changed
-                if '__cached_args_method_neighbors' in self.__dict__:
-                    ngb = self.__dict__['__cached_args_method_neighbors']
-                    for n in hs:
-                        try:
-                            del ngb[(n,)]
-                        except KeyError:
-                            pass
-                flush = False
-            # need hybridization recalculation
-            if '__cached_args_method_hybridization' in self.__dict__:
-                hyb = self.__dict__['__cached_args_method_hybridization']
+                ngb = self.__dict__['__cached_args_method_neighbors']
                 for n in hs:
                     try:
-                        del hyb[(n,)]
-                    except KeyError:  # already flushed before
+                        del ngb[(n,)]
+                    except KeyError:
                         pass
+                del self.__dict__['bonds_count']
+                flush = False
+            # need hybridization recalculation
+            hyb = self.__dict__['__cached_args_method_hybridization']
+            for n in hs:
+                try:
+                    del hyb[(n,)]
+                except KeyError:  # already flushed before
+                    pass
             for n in hs:  # hydrogens count recalculation
                 self._calc_implicit(n)
+            del self.__dict__['_cython_compiled_structure']
         return log
 
-    def __fix_resonance(self: Union['MoleculeContainer', 'StandardizeMolecule'], *, logging=False) -> \
-            Union[bool, List[int]]:
-        """
-        Transform biradical or dipole resonance structures into neutral form. Return True if structure form changed.
 
-        :param logging: return list of changed atoms.
-        """
-        atoms = self._atoms
-        charges = self._charges
-        radicals = self._radicals
-        bonds = self._bonds
-        entries, exits, rads, constrains = self.__entries()
-        hs = set()
-        while len(rads) > 1:
-            n = rads.pop()
-            for path in self.__find_delocalize_path(n, rads, constrains):
-                l, m, b = path[-1]
-                if b == 1:  # required pi-bond
-                    continue
-                try:
-                    atoms[m].valence_rules(charges[m], False, sum(int(y) for x, y in bonds[m].items() if x != l) + b)
-                except ValenceError:
-                    continue
-                radicals[n] = radicals[m] = False
-                rads.discard(m)
-                hs.add(n)
-                hs.update(x for _, x, _ in path)
-                for n, m, b in path:
-                    bonds[n][m]._Bond__order = b
-                break  # path found
-            # path not found. atom n keep as is
-        while entries and exits:
-            n = entries.pop()
-            for path in self.__find_delocalize_path(n, exits, constrains):
-                l, m, b = path[-1]
-                try:
-                    atoms[m].valence_rules(charges[m] - 1, radicals[m],
-                                           sum(int(y) for x, y in bonds[m].items() if x != l) + b)
-                except ValenceError:
-                    continue
-                charges[n] = charges[m] = 0
-                exits.discard(m)
-                hs.add(n)
-                hs.update(x for _, x, _ in path)
-                for n, m, b in path:
-                    bonds[n][m]._Bond__order = b
-                break  # path from negative atom to positive atom found.
-            # path not found. keep negative atom n as is
-        if hs:
-            for n in hs:
-                self._calc_implicit(n)
-            if logging:
-                return list(hs)
-            return True
-        if logging:
-            return []
-        return False
-
-    def __find_delocalize_path(self: 'MoleculeContainer', start, finish, constrains):
-        bonds = self._bonds
-        stack = [(start, n, 0, b.order + 1) for n, b in bonds[start].items() if n in constrains and b.order < 3]
-        path = []
-        seen = {start}
-        while stack:
-            last, current, depth, order = stack.pop()
-            if len(path) > depth:
-                seen.difference_update(x for _, x, _ in path[depth:])
-                path = path[:depth]
-
-            path.append((last, current, order))
-
-            if current in finish:
-                if depth:  # one bonded ignored. we search double bond transfer! A=A-A >> A-A=A.
-                    yield path
-                continue  # stop grow
-
-            depth += 1
-            seen.add(current)
-            diff = -1 if depth % 2 else 1
-            stack.extend((current, n, depth, bo) for n, b in bonds[current].items()
-                         if n not in seen and n in constrains and 1 <= (bo := b.order + diff) <= 3)
-
-    def __entries(self: 'MoleculeContainer'):
-        charges = self._charges
-        radicals = self._radicals
-        atoms = self._atoms
-        bonds = self._bonds
-
-        transfer = set()
-        entries = set()
-        exits = set()
-        rads = set()
-        for n, a in atoms.items():
-            if a.atomic_number not in {5, 6, 7, 8, 14, 15, 16, 33, 34, 52}:
-                # filter non-organic set, halogens and aromatics
-                continue
-            if charges[n] == -1:
-                if len(bonds[n]) == 4 and a.atomic_number == 5:  # skip boron
-                    continue
-                entries.add(n)
-            elif charges[n] == 1:
-                if len(bonds[n]) == 4 and a.atomic_number == 7:  # skip ammonia
-                    continue
-                exits.add(n)
-            elif radicals[n]:
-                rads.add(n)
-            transfer.add(n)
-        return entries, exits, rads, transfer
-
-
-__all__ = ['StandardizeMolecule']
+__all__ = ['Standardize']
