@@ -16,6 +16,7 @@
 #  You should have received a copy of the GNU Lesser General Public License
 #  along with this program; if not, see <https://www.gnu.org/licenses/>.
 #
+from array import array
 from CachedMethods import cached_args_method
 from collections import Counter, defaultdict
 from functools import cached_property
@@ -36,7 +37,7 @@ from ..algorithms.fingerprints import Fingerprints
 from ..algorithms.huckel import Huckel
 from ..algorithms.mcs import MCS
 from ..algorithms.smiles import MoleculeSmiles
-from ..algorithms.standardize import Saturation, StandardizeMolecule
+from ..algorithms.standardize import StandardizeMolecule
 from ..algorithms.stereo import MoleculeStereo
 from ..algorithms.tautomers import Tautomers
 from ..algorithms.x3dom import X3domMolecule
@@ -45,8 +46,7 @@ from ..periodictable import DynamicElement, Element, QueryElement
 
 
 class MoleculeContainer(MoleculeStereo, Graph[Element, Bond], Aromatize, StandardizeMolecule, MoleculeSmiles,
-                        DepictMolecule, Calculate2DMolecule, Fingerprints, Tautomers, MCS, Huckel,
-                        Saturation, X3domMolecule):
+                        DepictMolecule, Calculate2DMolecule, Fingerprints, Tautomers, MCS, Huckel, X3domMolecule):
     __slots__ = ('_conformers', '_atoms_stereo', '_hydrogens', '_cis_trans_stereo', '_allenes_stereo',
                  '_parsed_mapping', '_backup', '__meta', '__name')
 
@@ -676,17 +676,17 @@ class MoleculeContainer(MoleculeStereo, Graph[Element, Bond], Aromatize, Standar
             return dict(zip(so, oo))
         raise TypeError('MoleculeContainer expected')
 
-    def get_mapping(self, other: 'MoleculeContainer', **kwargs):
+    def get_mapping(self, other: 'MoleculeContainer', /, **kwargs):
         if isinstance(other, MoleculeContainer):
             return super().get_mapping(other, **kwargs)
         raise TypeError('MoleculeContainer expected')
 
-    def pack(self) -> bytes:
+    def pack(self, *, compressed=True) -> bytes:
         """
         Pack into compressed bytes.
         Note:
             * Less than 4096 atoms supported. Atoms mapping should be in range 1-4095.
-            * Implicit hydrogens count should be in range 0-7
+            * Implicit hydrogens count should be in range 0-6 or unspecified.
             * Isotope shift should be in range -15 - 15 relatively mdl.common_isotopes
             * Atoms neighbors should be in range 0-15
 
@@ -703,7 +703,7 @@ class MoleculeContainer(MoleculeStereo, Graph[Element, Bond], Aromatize, Standar
         5 bit - isotope (00000 - not specified, over = isotope - common_isotope + 16)
         7 bit - atomic number (<=118)
         32 bit - XY float16 coordinates
-        3 bit - hydrogens (0-7)
+        3 bit - hydrogens (0-7). Note: 7 == None
         4 bit - charge (charge + 4. possible range -4 - 4)
         1 bit - radical state
         Connection table: flatten list of neighbors. neighbors count stored in atom block.
@@ -716,6 +716,8 @@ class MoleculeContainer(MoleculeStereo, Graph[Element, Bond], Aromatize, Standar
         24 bit - atoms pair
         7 bit - zero padding. in future can be used for extra bond-level stereo, like atropoisomers.
         1 bit - sign
+
+        :param compressed: return zlib-compressed pack.
         """
         bonds = self._bonds
         if max(bonds) > 4095:
@@ -756,7 +758,10 @@ class MoleculeContainer(MoleculeStereo, Graph[Element, Bond], Aromatize, Standar
             hcr = (charges[n] + 4) << 1
             if radicals[n]:
                 hcr |= 1
-            hcr |= hydrogens[n] << 5
+            if (h := hydrogens[n]) is None:
+                hcr |= 224
+            else:
+                hcr |= h << 5
 
             # 2 bit tetrahedron sign | 2 bit - allene sign | 5 bit - isotope | 7 bit - atomic number (<=118)
             sia = a.atomic_number
@@ -796,19 +801,28 @@ class MoleculeContainer(MoleculeStereo, Graph[Element, Bond], Aromatize, Standar
         for o, ((n, m), s) in enumerate(cis_trans_stereo.items()):
             pack_into('>I', data, shift + 4 * o, (n << 20) | (m << 8) | s)
 
-        return compress(bytes(data), 9)
+        if compressed:
+            return compress(bytes(data), 9)
+        return bytes(data)
 
     @classmethod
-    def unpack(cls, data: bytes) -> 'MoleculeContainer':
+    def unpack(cls, data: bytes, /, *, compressed=True, _return_pack_length=False) -> 'MoleculeContainer':
         """
         Unpack from compressed bytes.
+
+        :param compressed: decompress data before processing.
         """
         try:  # windows? ;)
             from ._unpack import unpack
         except ImportError:
-            return cls.pure_unpack(data)
+            return cls.pure_unpack(data, compressed=compressed, _return_pack_length=_return_pack_length)
+        if compressed:
+            data = decompress(data)
+        if data[0] != 0:
+            raise ValueError('invalid pack header')
+
         (mapping, atom_numbers, isotopes, charges, radicals, hydrogens, plane, bonds,
-         atoms_stereo, allenes_stereo, cis_trans_stereo) = unpack(decompress(data))
+         atoms_stereo, allenes_stereo, cis_trans_stereo, pack_length) = unpack(data)
 
         mol = object.__new__(cls)
         mol._bonds = bonds
@@ -831,16 +845,23 @@ class MoleculeContainer(MoleculeStereo, Graph[Element, Bond], Aromatize, Standar
             a._Core__isotope = i
             a._graph = ref(mol)
             a._map = n
+        if _return_pack_length:
+            return mol, pack_length
         return mol
 
     @classmethod
-    def pure_unpack(cls, data: bytes) -> 'MoleculeContainer':
+    def pure_unpack(cls, data: bytes, /, *, compressed=True, _return_pack_length=False) -> 'MoleculeContainer':
         """
         Unpack from compressed bytes. Python implementation.
         """
         from ..files._mdl.mol import common_isotopes
+        if compressed:
+            data = memoryview(decompress(data))
+        elif not isinstance(data, memoryview):
+            data = memoryview(data)
+        if data[0] != 0:
+            raise ValueError('invalid pack header')
 
-        data = memoryview(decompress(data))
         mol = cls()
         atoms = mol._atoms
         bonds = mol._bonds
@@ -879,7 +900,7 @@ class MoleculeContainer(MoleculeStereo, Graph[Element, Bond], Aromatize, Standar
 
             charges[n] = ((hcr >> 1) & 0x0f) - 4
             radicals[n] = bool(hcr & 0x01)
-            hydrogens[n] = hcr >> 5
+            hydrogens[n] = None if (h := hcr >> 5) == 7 else h
             plane[n] = (x, y)
 
         bc = sum(neighbors.values()) // 2
@@ -916,6 +937,8 @@ class MoleculeContainer(MoleculeStereo, Graph[Element, Bond], Aromatize, Standar
         for o in range(acs & 0x0fff):  # cis/trans
             ct = unpack_from('>I', data, shift + 4 * o)[0]
             cis_trans_stereo[(ct >> 20, (ct >> 8) & 0x0fff)] = ct & 0x01
+        if _return_pack_length:
+            return mol, shift + (acs & 0x0fff) * 4
         return mol
 
     def _augmented_substructure(self, atoms: Iterable[int], deep: int):
@@ -950,6 +973,114 @@ class MoleculeContainer(MoleculeStereo, Graph[Element, Bond], Aromatize, Standar
         scope = set(self.connected_components[component])
         return {k: v & scope for k, v in self._screen_fingerprint.items() if not v.isdisjoint(scope)}
 
+    @cached_property
+    def _cython_compiled_structure(self):
+        # long I:
+        # bond: single, double, triple, aromatic, special = 5 bit
+        # atom: H-Ce: 58 bit
+        # transfer bit
+
+        # long II:
+        # atom Pr-Og: 60 bit
+        # hybridizations: 1-4 = 4 bit
+
+        # long III:
+        # isotope: not specified, isotope - common_isotope = -8 - +8 = 18 bit
+        # is_radical: 2 bit
+        # charge: -4 - +4: 9 bit
+        # implicit_hydrogens: 0-4 = 5 bit
+        # neighbors: 0-14 = 15 bit
+        # heteroatoms: 0-14 = 15 bit
+
+        # long IV:
+        # ring_sizes: not-in-ring bit, 3-atom ring, 4-...., 65-atom ring
+        from ..files._mdl.mol import common_isotopes
+
+        charges = self._charges
+        radicals = self._radicals
+        hydrogens = self._hydrogens
+        neighbors = self.neighbors
+        heteroatoms = self.heteroatoms
+        rings_sizes = self.atoms_rings_sizes
+        hybridization = self.hybridization
+
+        mapping = {}
+        numbers = []
+        bits1 = []
+        bits2 = []
+        bits3 = []
+        bits4 = []
+        for i, (n, a) in enumerate(self._atoms.items()):
+            mapping[n] = i
+            numbers.append(n)
+            v2 = 1 << (hybridization(n) - 1)
+            if (an := a.atomic_number) > 58:
+                v1 = 1  # transfer bit
+                v2 |= 1 << (122 - an)
+            else:
+                v1 = 1 << (59 - an)
+
+            if a.isotope:
+                v3 = 1 << (a.isotope - common_isotopes[a.atomic_symbol] + 54)
+                if radicals[n]:
+                    v3 |= 0x200000000000
+                else:
+                    v3 |= 0x100000000000
+            elif radicals[n]:
+                v3 = 0x8000200000000000
+            else:
+                v3 = 0x8000100000000000
+
+            v3 |= 1 << (charges[n] + 39)
+            v3 |= 1 << ((hydrogens[n] or 0) + 30)
+            v3 |= 1 << (neighbors(n) + 15)
+            v3 |= 1 << heteroatoms(n)
+
+            if n in rings_sizes:
+                v4 = 0
+                for r in rings_sizes[n]:
+                    if r > 65:  # big rings not supported
+                        continue
+                    v4 |= 1 << (65 - r)
+                if not v4:  # only 65+ rings. set as rings-free.
+                    v4 = 0x8000000000000000
+            else:  # not in rings
+                v4 = 0x8000000000000000
+
+            bits1.append(v1)
+            bits2.append(v2)
+            bits3.append(v3)
+            bits4.append(v4)
+
+        o_from = [0] * len(mapping)
+        o_to = [0] * len(mapping)
+        indices = [0] * self.bonds_count * 2
+        bonds = [0] * self.bonds_count * 2
+        start = 0
+        for n, ms in self._bonds.items():
+            i = mapping[n]
+            o_from[i] = start
+            for j, (m, b) in enumerate(ms.items(), start):
+                indices[j] = x = mapping[m]
+                v = bits1[x]
+                o = b.order
+                if o == 1:
+                    v |= 0x0800000000000000
+                elif o == 4:
+                    v |= 0x4000000000000000
+                elif o == 2:
+                    v |= 0x1000000000000000
+                elif o == 3:
+                    v |= 0x2000000000000000
+                else:
+                    v |= 0x8000000000000000
+                bonds[j] = v
+            start += len(ms)
+            o_to[i] = start
+
+        return (array('L', numbers), array('Q', bits1), array('Q', bits2), array('Q', bits3), array('Q', bits4),
+                array('Q', bonds), array('I', o_from), array('I', o_to), array('I', indices))
+
     def _calc_implicit(self, n: int):
         atoms = self._atoms
         atom = atoms[n]
